@@ -12,7 +12,7 @@ import string
 
 def truncateFiles(fileNameBase,snaps,zs,outputFileNameBase,numHalos,
                   haloFileExt = ".AHF_halos",profileFileExt = ".AHF_profiles",
-                  mtreeidxExt = ".AHF_mtree_idx", mtreeExt = ".AHF_mtree"):
+                  mtreeidxExt = ".AHF_mtree_idx", mtreeExt = ".AHF_mtree", skipmtree = False):
     """
     Truncates sets of AHF data files for a single cluster such as to shorten
     load times.
@@ -44,6 +44,9 @@ def truncateFiles(fileNameBase,snaps,zs,outputFileNameBase,numHalos,
         File extension to use for AHF_mtree_idx files
     mtreeExt : str, default = ".AHF_mtree"
         File extension to use for AHF_mtree files
+    skipmtree : bool, default = False
+        Whether to skip reading the .AHF_mtree file. Useful when the mtree
+        file is in the non-supported SUSSING-2013 format
 
     See Also
     --------
@@ -92,27 +95,39 @@ def truncateFiles(fileNameBase,snaps,zs,outputFileNameBase,numHalos,
 
         #truncate mtree_idx file
         try:
-            mtreeidxrows = np.genfromtxt(fileName + mtreeidxExt)
-            np.savetxt(outName + mtreeidxExt,mtreeidxrows[:numHalos,:],fmt='%d')
+            mtreeidxrows = np.genfromtxt(fileName + mtreeidxExt, max_rows = numHalos)
+
+            #check if mtreeidxrows is 1D and make it 2D if so
+            if mtreeidxrows.ndim == 1:
+                mtreeidxrows = np.array([mtreeidxrows])
+            np.savetxt(outName + mtreeidxExt,mtreeidxrows,fmt='%d')
             print("    mtree_idx done")
         except IOError:
             print("    WARNING: mtree_idx file not found")
 
         #truncate mtree file
-        try:
-            mtreerows = np.genfromtxt(fileName + mtreeExt)
-            #loop through rows to check how many rows need to be written to new file
-            haloCounter = 0
-            lineCounter = 0
-            while haloCounter < numHalos:
-                haloID, haloPart, numProg = mtreerows[lineCounter,:]
-                lineCounter += int(numProg)+1
-                haloCounter += 1
-            #lineCounter is now equal to the number of lines that we care about
-            np.savetxt(outName+mtreeExt,mtreerows[0:lineCounter,:],fmt='%d')
-            print("    mtree done")
-        except IOError:
-            print("    WARNING: mtree file not found")
+        if not skipmtree:
+            try:
+                mtreerows = np.genfromtxt(fileName + mtreeExt)
+                if len(mtreerows[0]) == 3:
+                    #non sussing format
+                    #loop through rows to check how many rows need to be written to new file
+                    haloCounter = 0
+                    lineCounter = 0
+                    while haloCounter < numHalos:
+                        haloID, haloPart, numProg = mtreerows[lineCounter,:]
+                        lineCounter += int(numProg)+1
+                        haloCounter += 1
+                    #lineCounter is now equal to the number of lines that we care about
+                    np.savetxt(outName+mtreeExt,mtreerows[0:lineCounter,:],fmt='%d')
+                    print("    mtree done")
+                else:
+                    #wrong format
+                    print("    WARNING: mtree is not correct format!")
+            except IOError:
+                print("    WARNING: mtree file not found")
+            except ValueError:
+                print("    WARNING: mtree is not of correct format! It may be in SUSSING-2013 format, which is not supported")
         print("Completed truncation: {0:.1f}%".format((i+1)/numFiles * 100))
 
 def getSnapNumToZMapGX(directory=""):
@@ -140,7 +155,13 @@ def getSnapNumToZMapGX(directory=""):
     files = os.listdir(directory)
     #assuming file name is of format GadgetX-NewMDCLUSTER_0001.snap_119.z0.221...
     #returns dictionary of {snapNo:z}
-    return {int(file[31:34]):float(file[36:41]) for file in files}
+    zDict = dict()
+    for file in files:
+        try:
+            zDict[int(file[31:34])] = float(file[36:41])
+        except ValueError:
+            pass
+    return zDict
 
 def getSnapNumToZMapGiz(directory=""):
     """
@@ -167,7 +188,13 @@ def getSnapNumToZMapGiz(directory=""):
     files = os.listdir(directory)
     #assuming file name is of format GIZMO-NewMDCLUSTER_0001.snap_119.z0.221...
     #returns dictionary of {snapNo:z}
-    return {int(file[29:32]):float(file[34:39]) for file in files}
+    zDict = dict()
+    for file in files:
+        try:
+            zDict[int(file[29:32])] = float(file[34:39])
+        except ValueError:
+            pass
+    return zDict
 
 def getMusZs(directory=""):
     """
@@ -194,13 +221,91 @@ def getMusZs(directory=""):
     """
     files = os.listdir(directory)
     #assuming file name is of format GadgetMUSIC-NewMDCLUSTER_0001.z0.000...
-    zs = [float(file[31:36]) for file in files]
-    #remove duplicates
-    zs = list(set(zs))
+    zs = set()
+    for file in files:
+        if file[30] == "z":
+            try:
+                zs.add( float(file[31:36]) )
+            except ValueError:
+                pass
+    zs = list(zs)
     #sort list such that in descending order
     zs.sort(reverse=True)
     return zs
 
+def getZfromFileName(fileName, suppress = False):
+    """
+    Lifts a redshift from a file name.
+
+    Assumes file names are formatted as "...zX.XXX..." where X.XXX is the
+    redshift.
+
+    Parameters
+    ----------
+    fileName : str
+    suppress : bool, default = False
+        Whether or not to suppress error messages. If no single redshift is
+        found and suppress is True, will return None
+
+    Returns
+    -------
+    z : float
+    """
+    #first we find where all of the "z" characters are in the name
+    zIndexes = [pos for pos, char in enumerate(fileName) if char == 'z']
+    zs = list()
+    for zIndex in zIndexes:
+        try:
+            zs.append( float(fileName[zIndex+1:zIndex+6]) )
+        except ValueError:
+            #bit of string after the z is not a number
+            pass
+    if len(zs) == 0:
+        msg = "Could not find a redshift in file name {0}\nPlease check the name is formatted as '...zX.XXX...'".format(fileName)
+        if not suppress:
+            raise ValueError(msg)
+        else:
+            return
+    elif len(zs) > 1:
+        msg = "Found multiple redshifts in file name {0}".format(fileName)
+        if not suppress:
+            raise ValueError(msg)
+        else:
+            return
+    return zs[0]
+
+def getZs(directory, num = -1):
+    """
+    Gets the redshifts of the snapshot files in a folder in descending order
+
+    Parameters
+    ----------
+    directory : str
+        The directory to search
+    num : int, optional
+        Specifies how many redshifts to return. Will return the first num
+        redshifts starting from redshift 0
+        If not specified will return all redshifts found
+
+    Returns
+    -------
+    zs : list of float
+        The redshifts of the files in the directory in descending order.
+    """
+    #get list of all redshifts in folder
+    files = os.listdir(directory)
+    zs = list(set([getZfromFileName(file, suppress = True) for file in files if getZfromFileName(file, suppress = True) != None]))
+    zs.sort(reverse=True)
+    #we want redshifts in descending order as snapNos should be ascending
+    #only want num redshifts if specified
+    if num != -1:
+        if num > len(zs):
+            msg = "more redshifts were asked for than the number found in directory {0}"\
+                  "\n{1} redshifts were found, and {2} were requested".format(directory,len(zs),num)
+            raise ValueError(msg)
+        zs = zs[-num:]
+    return zs
+
 class SafeFormatter(string.Formatter):
     """
     A string formatter used to preserve unused keys.
@@ -235,7 +340,7 @@ class SafeFormatter(string.Formatter):
         # finally call the default formatter
         return string.Formatter.vformat(self, format_string, args, kwargs)
 
-def loadClusters(clusterNums, snapNums, zs, simName, clusterFolderFmt = "NewMDCLUSTER_{clusterNum:0=4d}/",
+def loadClusters(clusterNums, snapNums, simName, clusterFolderFmt = "NewMDCLUSTER_{clusterNum:0=4d}/",
                  directory = "", fileBaseFmt = "{simName}-NewMDCLUSTER_{clusterNum:0=4d}.snap_{snap:0=3d}.z{z:.3f}",
                  profileExt=".AHF_profiles", haloExt=".AHF_halos",
                  mtreeidxExt=".AHF_mtree_idx", mtreeExt=".AHF_mtree", haloLimit=np.inf):
@@ -247,9 +352,9 @@ def loadClusters(clusterNums, snapNums, zs, simName, clusterFolderFmt = "NewMDCL
     clusterNums : list of int
         The list of cluster numbers to load
     snapNums : list of int
-        The list of snapshot numbers to load from each cluster
-    zs : list of float
-        The list of redshifts corresponding to each of the snapshots in snapNums
+        The list of snapshot numbers to load from each cluster in _ascending_
+        order. Should not contain any breaks and should include the snapshot
+        with z = 0.000
     simName : str
         The name of the simulation code to be inserted into file names
         (e.g. "GadgetX", "GadgetMUSIC")
@@ -322,7 +427,13 @@ def loadClusters(clusterNums, snapNums, zs, simName, clusterFolderFmt = "NewMDCL
 
     clusters = []
     for clusterNum in clusterNums:
-        fileBaseName = fmt.format(dir + clusterFolderFmt + fileBaseFmt,clusterNum=clusterNum,simName=simName)
+        #input relevant fields for file names
+        inputDir = fmt.format(directory + clusterFolderFmt,clusterNum=clusterNum,simName=simName)
+        fileBaseName = fmt.format(directory + clusterFolderFmt + fileBaseFmt,clusterNum=clusterNum,simName=simName)
+
+        #get redshifts in folder
+        zs = getZs(inputDir, num = len(snapNums))
+
         cluster = Cluster(fileBaseName, snapNums, zs, profileExt = profileExt,
                           haloExt = haloExt, mtreeidxExt = mtreeidxExt,
                           mtreeExt = mtreeExt, haloLimit = haloLimit,
@@ -331,11 +442,12 @@ def loadClusters(clusterNums, snapNums, zs, simName, clusterFolderFmt = "NewMDCL
 
     return clusters
 
-def truncateClusters(clusterNums, snapNums, zs, simName, haloLimit, outputDir,
+def truncateClusters(clusterNums, snapNums, simName, haloLimit, outputDir,
                      clusterFolderFmt = "NewMDCLUSTER_{clusterNum:0=4d}/",
                      directory = "", fileBaseFmt = "{simName}-NewMDCLUSTER_{clusterNum:0=4d}.snap_{snap:0=3d}.z{z:.3f}",
                      profileExt=".AHF_profiles", haloExt=".AHF_halos",
-                     mtreeidxExt=".AHF_mtree_idx", mtreeExt=".AHF_mtree"):
+                     mtreeidxExt=".AHF_mtree_idx", mtreeExt=".AHF_mtree",
+                     skipmtree = False):
     """
     Truncates files from multiple cluster folders at once
 
@@ -344,9 +456,9 @@ def truncateClusters(clusterNums, snapNums, zs, simName, haloLimit, outputDir,
     clusterNums : list of int
         The list of cluster numbers to load
     snapNums : list of int
-        The list of snapshot numbers to load from each cluster
-    zs : list of float
-        The list of redshifts corresponding to each of the snapshots in snapNums
+        The list of snapshot numbers to load from each cluster in _ascending_
+        order. Should not contain any breaks and should include the snapshot
+        with z = 0.000
     simName : str
         The name of the simulation code to be inserted into file names
         (e.g. "GadgetX", "GadgetMUSIC")
@@ -366,6 +478,9 @@ def truncateClusters(clusterNums, snapNums, zs, simName, haloLimit, outputDir,
         The format of the file name for the AHF files
         Defaults to "{simName}-NewMDCLUSTER_{clusterNum:0=4d}.snap
             _{snap:0=3d}.z{z:.3f}"
+    skipmtree : bool, default = False
+        Whether to skip reading the .AHF_mtree file. Useful if the mtree files
+        are in the non-supported SUSSING-2013 format
 
     Other Parameters
     ----------------
@@ -415,15 +530,30 @@ def truncateClusters(clusterNums, snapNums, zs, simName, haloLimit, outputDir,
     if not outputDir.endswith("/"):
         outputDir += "/"
 
+    numClusters = len(clusterNums)
     #using safe formatter to retain unused keys
     fmt = SafeFormatter()
 
-    for clusterNum in clusterNums:
+    for i, clusterNum in enumerate(clusterNums):
         #format file base name with cluster number and simulation name
-        fileBaseName = fmt.format(dir + clusterFolderFmt + fileBaseFmt,clusterNum=clusterNum,simName=simName)
+        inputDir = fmt.format(directory + clusterFolderFmt,clusterNum=clusterNum,simName=simName)
+        fileBaseName = fmt.format(directory + clusterFolderFmt + fileBaseFmt,clusterNum=clusterNum,simName=simName)
         outputFileNameBase = fmt.format(outputDir + clusterFolderFmt + fileBaseFmt,clusterNum=clusterNum,simName=simName)
 
+        outputClusterDir = fmt.format(outputDir + clusterFolderFmt, clusterNum=clusterNum, simName=simName)
+        outputDirFmtted = fmt.format(outputDir, clusterNum=clusterNum, simName=simName)
+
+        zs = getZs(inputDir, num = len(snapNums))
+
+        #check if output files exist already, if not create them
+        if not os.path.isdir(outputDirFmtted):
+            os.mkdir(outputDirFmtted)
+        if not os.path.isdir(outputClusterDir):
+            os.mkdir(outputClusterDir)
+
         #truncate files for cluster
         truncateFiles(fileBaseName,snapNums,zs,outputFileNameBase,haloLimit,
                           haloFileExt = haloExt,profileFileExt = profileExt,
-                          mtreeidxExt = mtreeidxExt, mtreeExt = mtreeExt)
+                          mtreeidxExt = mtreeidxExt, mtreeExt = mtreeExt,
+                          skipmtree = skipmtree)
+        print("\n===CLUSTER {0} DONE===    Overall Completion: {1:.1f}%\n".format(clusterNum, (i+1)/numClusters * 100))
